from enum import Enum


class SharingType(Enum):
    UNLIMITED = "UNLIMITED"
    PERMISSIONS_LIMITED = "PERMISSIONS_LIMITED"
    DISABLED = "DISABLED"


class ObjectType(Enum):
    ARTIFACT = "ARTIFACT"
    FILE = "FILE"
    FOLDER = "FOLDER"


class FolderType(Enum):
    STANDARD = "STANDARD"
    SHARES = "SHARES"
    PERSONAL = "PERSONAL"


class ErrorCode(Enum):
    ARTIFACT_NOT_FOUND = "ARTIFACT_NOT_FOUND"
    FILE_GONE = "FILE_GONE"
    FILE_NOT_FOUND = "FILE_NOT_FOUND"
    FOLDER_GONE = "FOLDER_GONE"
    FOLDER_NOT_FOUND = "FOLDER_NOT_FOUND"
    FORBIDDEN = "FORBIDDEN"
    UNPROCESSABLE_ENTITY = "UNPROCESSABLE_ENTITY"
